#include "stddef.h"
#include "stdint.h"
#include "stdbool.h"
#include "stdlib.h"
#include "math.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"

#include "GatewayFSM.h"
#include "SensorFSM.h"
#include "pwr_gpio.h"


#include "WIT2FSM.h"

static gw_state_t eState = GW_STATE_IDLE;
static uint32_t u32CollectStartTick = 0;
static uint32_t u32MotorSyncStartTick = 0;

extern sensorFSM_t sensors[6];
extern speedSyncFSM spdSyncFSM;

void vGatewayInit(void)
{
    eState = GW_STATE_IDLE;
    u32CollectStartTick = 0;
    u32MotorSyncStartTick = 0;
}

void vGatewayProcess(void)
{
    uint32_t tu32_deviceRun = 0;
    uint32_t tu32_ret = 0;
    uint32_t tu32_completedCount = 0;
    switch (eState)
    {
    case GW_STATE_IDLE:
    {
        tu32_deviceRun = WIT2FSM_DeviceRunCheck();
        if (tu32_deviceRun == true)
        {
            if (WIT2FSM_NeedSend() == true)
            {
                tu32_ret = WIT2FSM_StartCollect();
                if (tu32_ret == true)
                {
                    eState = GW_STATE_COLLECTING;
                    spdSyncFSM.enable = WIT2FSM_GetSpdSyncConfig();
                    u32CollectStartTick = xTaskGetTickCount();
                }
                else
                {
					// Start collect failed  因为modbus 广播不需要返回，所以这里不会进来
					eState = GW_STATE_COMPLETED;
                }
            }
            else
            {
                // Start collect success
                if (WIT2FSM_AmlVibCheck() == true)
                {
                    // Has anomaly vibration
                    tu32_ret = WIT2FSM_StartCollect();
                    if (tu32_ret == true)
                    {
                        WIT2FSM_SendAmlVibAlarm();
                        eState = GW_STATE_COLLECTING;
                        spdSyncFSM.enable = false;
                        u32CollectStartTick = xTaskGetTickCount();
                    }
                    else
                    {
                        // Start collect failed
						// 因为modbus 广播不需要返回，所以这里不会进来
						eState = GW_STATE_COMPLETED;
                    }
                }
//                else
//                {
//                    // Has not anomaly vibration
//                    WIT2FSM_PT100Read();
//                }
            }
        }
        else
        {
            // Device not run
            // Send Gateway feature addr=0
            if (WIT2FSM_SendHeartToServer(0) == true)
            {
                // Send success

                break;
            }
            else
            {
                // Send failed
            }
        }
        WIT2FSM_SendError();
        vTaskDelay(10);
    }
    break;
    case GW_STATE_COLLECTING:
    {
        if (abs((long long)xTaskGetTickCount() - (long long)u32CollectStartTick) > TIMEOUT_COLLECT_TICK_MAX)
        {
            if (spdSyncFSM.enable == true)
            {
                if (spdSyncFSM.mpsAddr == 0 || spdSyncFSM.mpsAddr > 6)
                {
                    eState = GW_STATE_COMPLETED;
                }
                else if (sensors[spdSyncFSM.mpsAddr - 1].state != SEN_STATE_COMPLETED)
                {
                    eState = GW_STATE_COMPLETED;
                }
                else
                {
                    eState = GW_STATE_MOTORSYNC;
                }
            }
            else
            {
                eState = GW_STATE_COMPLETED;
            }
            break;
        }

        tu32_completedCount = 0;
        for (size_t i = 0; i < SENSOR_NUMBER_MAX; i++)
        {
            if (sensors[i].available != true)
            {
                ++tu32_completedCount;
                continue;
            }

            if (sensors[i].state == SEN_STATE_COLLECTING)
            {
                // ready
            }
            else if (sensors[i].state == SEN_STATE_COMPLETED)
            {
                ++tu32_completedCount;
            }
            else
            {
            }
        }

        if (tu32_completedCount == SENSOR_NUMBER_MAX)
        {
            // Sensor collect complete
            if (spdSyncFSM.enable == true)
            {
                eState = GW_STATE_MOTORSYNC;
            }
            else
            {
                eState = GW_STATE_COMPLETED;
            }
        }
    }
    break;
    case GW_STATE_MOTORSYNC:
    {
        if (spdSyncFSM.mpsAddr == 0 || spdSyncFSM.mpsAddr > 6)
        {
            eState = GW_STATE_COMPLETED;
        }

        else if (sensors[spdSyncFSM.mpsAddr].available == true)
        {
            // motor speed sync
            if (WIT2FSM_MotorSpeedSync(&spdSyncFSM) == true)
            {
                u32MotorSyncStartTick = xTaskGetTickCount();
                eState = GW_STATE_WAITFOR_MOTORSYNC;
            }
            else
            {
                // motor speed sync failed
                eState = GW_STATE_COMPLETED;
            }
        }

        else{
            eState = GW_STATE_COMPLETED;
        }
    }
    break;
    case GW_STATE_WAITFOR_MOTORSYNC:
    {
        if (abs((long long)xTaskGetTickCount() - (long long)u32MotorSyncStartTick) > TIMEOUT_MOTORSYNC_TICK_MAX)
        {
            // Motor sync timeout
					  eState = GW_STATE_COMPLETED;
            break;
        }

        tu32_completedCount = 0;
        for (size_t i = 0; i < SENSOR_NUMBER_MAX; i++)
        {
            if (sensors[i].available != true)
            {
                ++tu32_completedCount;
                continue;
            }

            if (sensors[i].state == SEN_STATE_RECALCULATE) // 老传感器没有转速同步
            {
                // ready
            }
            else if (sensors[i].state == SEN_STATE_COMPLETED)
            {
                ++tu32_completedCount;
            }
            else
            {
                // unknow state  依靠超时
            }
        }

        if (tu32_completedCount == SENSOR_NUMBER_MAX)
        {
            eState = GW_STATE_COMPLETED;
        }
    }
    break;
    case GW_STATE_COMPLETED:
    {
        bool needResetSensor = false;
        // Send Gateway feature addr=0
        if (WIT2FSM_SendGatewayFeatureToServer(0) == true)
        {
            // Send success
        }
        else
        {
            // Send failed
        }

        for (size_t i = 0; i < SENSOR_NUMBER_MAX; i++)
        {
            if (sensors[i].available != true)
            {
                continue;
            }

            if (sensors[i].state != SEN_STATE_COMPLETED)
            {
                sensors[i].u32SensorStateErrCnt++;
                needResetSensor = true;
                continue;
            }
            else
            {
                sensors[i].u32SensorStateErrCnt = 0;
            }
            

            // Send Sensor feature
            if (WIT2FSM_SendSensorFeatureToServer(0, i + 1) == true)
            {
                // Send success
            }
            else
            {
                // Send failed
            }

            // Send Sensor data
            if (WIT2FSM_SendWaveToServer(0, i + 1) == true)
            {
                // Send success
            }
            else
            {
                // Send failed
            }
        }

        // Clear variable
        WIT2FSM_ClearVarible();
        if (needResetSensor == true)
        {
            for (size_t i = 0; i < SENSOR_NUMBER_MAX; i++)
            {
                if (sensors[i].available == false)
                {
                    continue;
                }
                if (sensors[i].u32SensorStateErrCnt == 0)
                {
                    continue;
                }

                if (sensors[i].u32SensorStateErrCnt < 11)
                {
                    // reset sensor
                    pwr_set(false, true);
                    vTaskDelay(500);
                    pwr_set(true, true);
                    vTaskDelay(2000);
                }
                else
                {
                    // disable
                    sensors[i].available = false;
                    sensors[i].errLastUpdateTick = xTaskGetTickCount();
                }
            }
        }

        eState = GW_STATE_IDLE;
    }
    break;
    case GW_STATE_ERROR:
    {
        eState = GW_STATE_IDLE;
			vTaskDelay(2000);
    }
    break;
    default:
    {
        eState = GW_STATE_IDLE;
			vTaskDelay(2000);
    }
        break;
    }
}

gw_state_t xGatewayFSMStateGet(void)
{
    return eState;
}
