/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include "algorithm.h"

#define ALGORITHM_HANDLE_SIZE			sizeof(ALGORITHM_HANDLE_TYPE)
#define ALGORITHM_COMPLEX_SIZE			sizeof(Complex)
#if ALGORITHM_EXT_MEMORY_ACCELERATE > 0
#define ALGORITHM_ACC_MAX_HANDLE_SIZE	(ACCELERATE_BUFFER_SIZE * ALGORITHM_COMPLEX_SIZE / ALGORITHM_HANDLE_SIZE)
static Complex g_xAccelerateLeftBuffer[ACCELERATE_BUFFER_SIZE];
static Complex g_xAccelerateRightBuffer[ACCELERATE_BUFFER_SIZE];
#endif

/* Private functions ---------------------------------------------------------*/

/**
  * @brief  计算数据平均值
  * @param  pxData: 输入数据数组(直接内存中计算，内存与外部计算选其中一种)
  * @param  ulSize: 输入数据大小
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE Mean(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress)
{
	ALGORITHM_RESULT_TYPE result = 0;
	int i;
	
	if (pxData == NULL && pFuncReadData == NULL)
		return 0;
	
	if (pxData != NULL)
	{
		for (i = 0;i < ulSize;i++)
		{			
			result += pxData[i] * 1.0f / ulSize;
		}
	}
	else
	{
		ALGORITHM_HANDLE_TYPE xTempData;
		
		for (i = 0;i < ulSize;i++)
		{
		#if ALGORITHM_EXT_MEMORY_ACCELERATE > 0
			if (i % ALGORITHM_ACC_MAX_HANDLE_SIZE == 0)
			{
				pFuncReadData(ulReadAddress + i * ALGORITHM_HANDLE_SIZE, (unsigned char *)g_xAccelerateLeftBuffer, 
					(ulSize - i) > ALGORITHM_ACC_MAX_HANDLE_SIZE ? 
					ALGORITHM_ACC_MAX_HANDLE_SIZE * ALGORITHM_HANDLE_SIZE : 
					(ulSize - i) * ALGORITHM_HANDLE_SIZE);
			}
			xTempData = ((float *)g_xAccelerateLeftBuffer)[i % ALGORITHM_ACC_MAX_HANDLE_SIZE];
		#else
			pFuncReadData(ulReadAddress + i * ALGORITHM_HANDLE_SIZE, (unsigned char *)&xTempData, ALGORITHM_HANDLE_SIZE);
		#endif
			result += xTempData * 1.0f / ulSize;
		}
	}
	
	return result;
}

/**
  * @brief  计算数据均方根
  * @param  pxData: 输入数据数组
  * @param  ulSize: 输入数据大小
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE RMS(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress)
{
	ALGORITHM_RESULT_TYPE result = 0;
	int i;
	
	if (pxData == NULL && pFuncReadData == NULL)
		return 0;
	
	if (pxData != NULL)
	{
		for (i = 0;i < ulSize;i++)
		{
			result += pxData[i] * pxData[i] * 1.0f / ulSize;
		}

	}
	else
	{
		ALGORITHM_HANDLE_TYPE xTempData;

		for (i = 0;i < ulSize;i++)
		{
		#if ALGORITHM_EXT_MEMORY_ACCELERATE > 0
			if (i % ALGORITHM_ACC_MAX_HANDLE_SIZE == 0)
			{
				pFuncReadData(ulReadAddress + i * ALGORITHM_HANDLE_SIZE, (unsigned char *)g_xAccelerateLeftBuffer, 
					(ulSize - i) > ALGORITHM_ACC_MAX_HANDLE_SIZE ? 
					ALGORITHM_ACC_MAX_HANDLE_SIZE * ALGORITHM_HANDLE_SIZE : 
					(ulSize - i) * ALGORITHM_HANDLE_SIZE);
			}
			xTempData = ((float *)g_xAccelerateLeftBuffer)[i % ALGORITHM_ACC_MAX_HANDLE_SIZE];
		#else
			pFuncReadData(ulReadAddress + i * ALGORITHM_HANDLE_SIZE, (unsigned char *)&xTempData, ALGORITHM_HANDLE_SIZE);
		#endif
			result += xTempData * xTempData * 1.0f / ulSize;
		}
	}
	
	return sqrt(result);
}

/**
  * @brief  计算数据标准差
  * @param  pxData: 输入数据数组
  * @param  ulSize: 输入数据大小
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE Std(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress)
{
	ALGORITHM_RESULT_TYPE result = 0;
	ALGORITHM_RESULT_TYPE mean = 0;
	int i;
	
	if (pxData == NULL && pFuncReadData == NULL)
		return 0;
	
	mean = Mean(pxData, ulSize, pFuncReadData, ulReadAddress);
	
	if (pxData != NULL)
	{
		for (i = 0;i < ulSize;i++)
		{
			result += (pxData[i] - mean) * (pxData[i] - mean) * 1.0f / ulSize;
		}
	}
	else
	{
		ALGORITHM_HANDLE_TYPE xTempData;
		
		for (i = 0;i < ulSize;i++)
		{
		#if ALGORITHM_EXT_MEMORY_ACCELERATE > 0
			if (i % ALGORITHM_ACC_MAX_HANDLE_SIZE == 0)
			{
				pFuncReadData(ulReadAddress + i * ALGORITHM_HANDLE_SIZE, (unsigned char *)g_xAccelerateLeftBuffer, 
					(ulSize - i) > ALGORITHM_ACC_MAX_HANDLE_SIZE ? 
					ALGORITHM_ACC_MAX_HANDLE_SIZE * ALGORITHM_HANDLE_SIZE : 
					(ulSize - i) * ALGORITHM_HANDLE_SIZE);
			}
			xTempData = ((float *)g_xAccelerateLeftBuffer)[i % ALGORITHM_ACC_MAX_HANDLE_SIZE];
		#else
			pFuncReadData(ulReadAddress + i * ALGORITHM_HANDLE_SIZE, (unsigned char *)&xTempData, ALGORITHM_HANDLE_SIZE);
		#endif
			result += (xTempData - mean) * (xTempData - mean) * 1.0f / ulSize;
		}
	}
	
	return sqrt(result);
}

/**
  * @brief  计算数据方差
  * @param  pxData: 输入数据数组
  * @param  ulSize: 输入数据大小
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE Var(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress)
{
	ALGORITHM_RESULT_TYPE result = 0;
	ALGORITHM_RESULT_TYPE std = 0;
	
	std = Std(pxData, ulSize, pFuncReadData, ulReadAddress);
	
	result = std * std;
	
	return result;
}

/**
  * @brief  计算数据最大值
  * @param  pxData: 输入数据数组
  * @param  ulSize: 输入数据大小
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE Max(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress)
{
	ALGORITHM_RESULT_TYPE result = 0;
	int i;
	
	if (pxData == NULL && pFuncReadData == NULL)
		return 0;
	
	if (pxData != NULL)
	{
		result = pxData[0];
		
		for (i = 1;i < ulSize;i++)
		{
			if (pxData[i] > result)
				result = pxData[i];
		}
	}
	else
	{
		ALGORITHM_HANDLE_TYPE xTempData;
		
		pFuncReadData(ulReadAddress, (unsigned char *)&xTempData, ALGORITHM_HANDLE_SIZE);
		result = xTempData;
		
		for (i = 1;i < ulSize;i++)
		{
		#if ALGORITHM_EXT_MEMORY_ACCELERATE > 0
			if (i % ALGORITHM_ACC_MAX_HANDLE_SIZE == 0 || i == 1)
			{
				int index = i;
				
				if (i == 1)
					index = 0;
				else
					index = i;
				
				pFuncReadData(ulReadAddress + index * ALGORITHM_HANDLE_SIZE, (unsigned char *)g_xAccelerateLeftBuffer, 
					(ulSize - index) > ALGORITHM_ACC_MAX_HANDLE_SIZE ? 
					ALGORITHM_ACC_MAX_HANDLE_SIZE * ALGORITHM_HANDLE_SIZE : 
					(ulSize - index) * ALGORITHM_HANDLE_SIZE);
			}
			xTempData = ((float *)g_xAccelerateLeftBuffer)[i % ALGORITHM_ACC_MAX_HANDLE_SIZE];
		#else
			pFuncReadData(ulReadAddress + i * ALGORITHM_HANDLE_SIZE, (unsigned char *)&xTempData, ALGORITHM_HANDLE_SIZE);
		#endif
			if (xTempData > result)
				result = xTempData;
		}
	}
	
	return result;	
}

/**
  * @brief  计算数据最小值
  * @param  pxData: 输入数据数组
  * @param  ulSize: 输入数据大小
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE Min(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize, 
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress)
{
	ALGORITHM_RESULT_TYPE result = 0;
	int i;
	
	if (pxData == NULL && pFuncReadData == NULL)
		return 0;
	
	if (pxData != NULL)
	{
		result = pxData[0];
		
		for (i = 1;i < ulSize;i++)
		{
			if (pxData[i] < result)
				result = pxData[i];
		}
	}
	else
	{
		ALGORITHM_HANDLE_TYPE xTempData;
		
		pFuncReadData(ulReadAddress, (unsigned char *)&xTempData, ALGORITHM_HANDLE_SIZE);
		result = xTempData;
		
		for (i = 1;i < ulSize;i++)
		{
		#if ALGORITHM_EXT_MEMORY_ACCELERATE > 0
			if (i % ALGORITHM_ACC_MAX_HANDLE_SIZE == 0 || i == 1)
			{
				int index = i;
				
				if (i == 1)
					index = 0;
				else
					index = i;
				
				pFuncReadData(ulReadAddress + index * ALGORITHM_HANDLE_SIZE, (unsigned char *)g_xAccelerateLeftBuffer, 
					(ulSize - index) > ALGORITHM_ACC_MAX_HANDLE_SIZE ? 
					ALGORITHM_ACC_MAX_HANDLE_SIZE * ALGORITHM_HANDLE_SIZE : 
					(ulSize - index) * ALGORITHM_HANDLE_SIZE);
			}
			xTempData = ((float *)g_xAccelerateLeftBuffer)[i % ALGORITHM_ACC_MAX_HANDLE_SIZE];
		#else
			pFuncReadData(ulReadAddress + i * ALGORITHM_HANDLE_SIZE, (unsigned char *)&xTempData, ALGORITHM_HANDLE_SIZE);
		#endif
			if (xTempData < result)
				result = xTempData;
		}
	}
	
	return result;	
}

/**
  * @brief  计算数据峭度
  * @param  pxData: 输入数据数组
  * @param  ulSize: 输入数据大小
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE Kurtosis(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize, 
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress)
{
	ALGORITHM_RESULT_TYPE mean, std, result = 0;
	int i;
	
	std = Std(pxData, ulSize, pFuncReadData, ulReadAddress);
	
	if (std != 0)
	{
		mean = Mean(pxData, ulSize, pFuncReadData, ulReadAddress);
	
		if (pxData != NULL)
		{
			for (i = 0;i < ulSize;i++)
			{
				result += powf((pxData[i] - mean) * 1.0f / std, 4) / ulSize;
			}
		}
		else
		{
			ALGORITHM_HANDLE_TYPE xTempData;
			
			for (i = 0;i < ulSize;i++)
			{
			#if ALGORITHM_EXT_MEMORY_ACCELERATE > 0
				if (i % ALGORITHM_ACC_MAX_HANDLE_SIZE == 0)
				{
					pFuncReadData(ulReadAddress + i * ALGORITHM_HANDLE_SIZE, (unsigned char *)g_xAccelerateLeftBuffer, 
						(ulSize - i) > ALGORITHM_ACC_MAX_HANDLE_SIZE ? 
						ALGORITHM_ACC_MAX_HANDLE_SIZE * ALGORITHM_HANDLE_SIZE : 
						(ulSize - i) * ALGORITHM_HANDLE_SIZE);
				}
				xTempData = ((float *)g_xAccelerateLeftBuffer)[i % ALGORITHM_ACC_MAX_HANDLE_SIZE];
			#else
				pFuncReadData(ulReadAddress + i * ALGORITHM_HANDLE_SIZE, (unsigned char *)&xTempData, ALGORITHM_HANDLE_SIZE);
			#endif
				result += powf((xTempData - mean) * 1.0f / std, 4) / ulSize;
			}
		}
	}
	else
	{
		return 3;
	}
	
	return result;
}

//ALGORITHM_RESULT_TYPE DFT(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize, ALGORITHM_HANDLE_TYPE *pxOut,
//							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress,
//						    void (*pFuncWriteData)(unsigned long, unsigned char *, unsigned long), unsigned long ulWriteAddress)
//{
//	int k, n, N;
//	ALGORITHM_HANDLE_TYPE real = 0, image = 0;
//	
//	if (pxData == NULL && pFuncReadData == NULL)
//		return 0;
//	
//	N = ulSize;
//	if (pxData != NULL)
//	{			
//		if (pxOut == NULL)
//			return 0;
//		
//		for (k = 0;k < N / 2;k++)
//		{
//			real = 0;
//			image = 0;
//			for (n = 0;n < N;n++)
//			{
//				real += cos(2 * ALGORITHM_PI / N * n * k) * pxData[n];
//				image += -sin(2 * ALGORITHM_PI / N * n * k) * pxData[n];
//			}
//			
//			pxOut[k] = sqrt(real * real + image * image);
//		}
//	}
//	else
//	{
//		if (pFuncWriteData == NULL)
//			return 0;
//		
//		for (k = 0;k < N / 2;k++)
//		{
//			ALGORITHM_HANDLE_TYPE xTempData, result;
//			
//			real = 0;
//			image = 0;
//			pFuncReadData(ulReadAddress + k * ALGORITHM_HANDLE_SIZE, (unsigned char *)&xTempData, ALGORITHM_HANDLE_SIZE);
//			
//			for (n = 0;n < N;n++)
//			{
//				real += cos(2 * ALGORITHM_PI / N * n * k) * xTempData;
//				image += -sin(2 * ALGORITHM_PI / N * n * k) * xTempData;
//			}
//			
//			result = sqrt(real * real + image * image);
//			pFuncWriteData(ulWriteAddress + k * ALGORITHM_HANDLE_SIZE, (unsigned char *)&result, ALGORITHM_HANDLE_SIZE);
//		}
//	}
//	
//	return 1;
//}

//ALGORITHM_RESULT_TYPE DFT_GetOnePoint(unsigned long k, ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize,
//							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress)
//{
//	int n, N;
//	ALGORITHM_HANDLE_TYPE real = 0, image = 0, result;
//	
//	if (pxData == NULL && pFuncReadData == NULL)
//		return 0;
//	
//	N = ulSize;
//	if (pxData != NULL)
//	{			
//		for (n = 0;n < N;n++)
//		{
//			real += cos(2 * ALGORITHM_PI / N * n * k) * pxData[n];
//			image += -sin(2 * ALGORITHM_PI / N * n * k) * pxData[n];
//		}
//	}
//	else
//	{
//		ALGORITHM_HANDLE_TYPE xTempData;
//		
//		pFuncReadData(ulReadAddress + k * ALGORITHM_HANDLE_SIZE, (unsigned char *)&xTempData, ALGORITHM_HANDLE_SIZE);
//		
//		for (n = 0;n < N;n++)
//		{
//			real += cos(2 * ALGORITHM_PI / N * n * k) * xTempData;
//			image += -sin(2 * ALGORITHM_PI / N * n * k) * xTempData;
//		}
//	}
//	
//	result = sqrt(real * real + image * image);
//	
//	return result;
//}

/**
  * @brief  对数据进行FFT变换
  * @param  pxData: 输入实数数据数组
  * @param  ulSize: 输入数据大小
  * @param  pxOut: 输出傅里叶变换后的结果，大小必须未输入数组的两倍
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @param  pFuncWriteData: 函数指针->外部内存写函数(用于大量数据计算时)
  * @param  ulWriteAddress: 外部内存写开始地址
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE RFFT(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize, Complex *pxOut, 
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress,
							void (*pFuncWriteData)(unsigned long, unsigned char *, unsigned long), unsigned long ulWriteAddress)
{
	int i, j, k;
	int depth = log2(ulSize);
	int bitReverse, levelCount;
	Complex W, xTempComplex;
	ALGORITHM_HANDLE_TYPE xTempData;
	
	//FFT 公式： 前半部分 x(k) = E(k) + W * O(k)
	//           后半部分 x(k) = E(k) - W * O(k)
	if (pxData == NULL && pFuncReadData == NULL)
		return 0;
	
	if (pxData != NULL)
	{
		Complex *pxOutComplex;
		
		if (pxOut == NULL)
			return 0;
		
		pxOutComplex = (Complex *)pxOut;
		
		//序列倒序转换
		for (i = 0;i < ulSize;i++)
		{
			bitReverse = 0;
			for (j = 0;j < depth;j++)
			{
				if (i & (1 << j))
				{
					bitReverse |= (1 << (depth - j - 1));
				}
			}
			
			if (bitReverse >= i)
			{
				pxOutComplex[i].fReal = pxData[bitReverse];
				pxOutComplex[i].fImage = 0;
				pxOutComplex[bitReverse].fReal = pxData[i];
				pxOutComplex[bitReverse].fImage = 0;
			}
		}
		
		//从下往上推导
		for (i = 0;i < depth;i++)
		{
			//每一级所拥有的组数
			levelCount = (1 << i);
			for (j = 0;j < ulSize;j += 2 * levelCount)
			{
				//对每一组进行蝶形运算
				for (k = 0;k < levelCount;k++)
				{	
					W.fReal = cos(2 * ALGORITHM_PI / ulSize * (ulSize * k / 2 / levelCount));
					W.fImage = -sin(2 * ALGORITHM_PI / ulSize * (ulSize * k / 2 / levelCount));
					
					xTempComplex = Complex_Mul(&W, &pxOutComplex[j + k + levelCount]);
					//Sub
					pxOutComplex[j + k + levelCount] = Complex_Sub(&pxOutComplex[j + k], &xTempComplex);
					//Add
					pxOutComplex[j + k] = Complex_Add(&pxOutComplex[j + k], &xTempComplex);
				}
			}
		}
	}
	else
	{
	#if !ALGORITHM_EXT_MEMORY_ACCELERATE > 0
		Complex xCalComplex1, xCalComplex2;
	#endif
		
		if (pFuncWriteData == NULL)
			return 0;
		
		//序列倒序转换
		for (i = 0;i < ulSize;i++)
		{
			bitReverse = 0;
			for (j = 0;j < depth;j++)
			{
				if (i & (1 << j))
				{
					bitReverse |= (1 << (depth - j - 1));
				}
			}

			if (bitReverse >= i)
			{
				//out[bitReverse] = in[i]
				pFuncReadData(ulReadAddress + i * sizeof(ALGORITHM_HANDLE_TYPE), (unsigned char *)&xTempData, sizeof(ALGORITHM_HANDLE_TYPE)); 
				xTempComplex.fReal = xTempData;
				xTempComplex.fImage = 0;
				pFuncWriteData(ulWriteAddress + bitReverse * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xTempComplex, ALGORITHM_COMPLEX_SIZE);
				
				//out[i] = in[bitReverse]
				pFuncReadData(ulReadAddress + bitReverse * sizeof(ALGORITHM_HANDLE_TYPE), (unsigned char *)&xTempData, sizeof(ALGORITHM_HANDLE_TYPE)); 
				xTempComplex.fReal = xTempData;
				xTempComplex.fImage = 0;
				pFuncWriteData(ulWriteAddress + i * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xTempComplex, ALGORITHM_COMPLEX_SIZE);
			}
		}
		
		//从下往上推导
		for (i = 0;i < depth;i++)
		{
			//每一级所拥有的组数
			levelCount = (1 << i);
			for (j = 0;j < ulSize;j += 2 * levelCount)
			{	
				//对每一组进行蝶形运算
				for (k = 0;k < levelCount;k++)
				{	
					W.fReal = cos(2 * ALGORITHM_PI / ulSize * (ulSize * k / 2 / levelCount));
					W.fImage = -sin(2 * ALGORITHM_PI / ulSize * (ulSize * k / 2 / levelCount));
				#if ALGORITHM_EXT_MEMORY_ACCELERATE > 0
					if (k % ACCELERATE_BUFFER_SIZE == 0)
					{
						if (k != 0)
						{
							pFuncWriteData(ulWriteAddress + (j + k - ACCELERATE_BUFFER_SIZE) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateLeftBuffer, ACCELERATE_BUFFER_SIZE * ALGORITHM_COMPLEX_SIZE);
							pFuncWriteData(ulWriteAddress + (j + k - ACCELERATE_BUFFER_SIZE + levelCount) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateRightBuffer, ACCELERATE_BUFFER_SIZE * ALGORITHM_COMPLEX_SIZE);
						}
						//left
						pFuncReadData(ulWriteAddress + (j + k) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateLeftBuffer, (levelCount > ACCELERATE_BUFFER_SIZE ? ACCELERATE_BUFFER_SIZE : levelCount) * ALGORITHM_COMPLEX_SIZE);
						//right
						pFuncReadData(ulWriteAddress + (j + k + levelCount) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateRightBuffer, (levelCount > ACCELERATE_BUFFER_SIZE ? ACCELERATE_BUFFER_SIZE : levelCount) * ALGORITHM_COMPLEX_SIZE);
					}
					//W * O(k)
					xTempComplex = Complex_Mul(&W, &g_xAccelerateRightBuffer[k % ACCELERATE_BUFFER_SIZE]);
					//Sub  E(k) - W * O(k)
					g_xAccelerateRightBuffer[k % ACCELERATE_BUFFER_SIZE] = Complex_Sub(&g_xAccelerateLeftBuffer[k % ACCELERATE_BUFFER_SIZE], &xTempComplex);
					
					//Add E(k) + W * O(k)
					g_xAccelerateLeftBuffer[k % ACCELERATE_BUFFER_SIZE] = Complex_Add(&g_xAccelerateLeftBuffer[k % ACCELERATE_BUFFER_SIZE], &xTempComplex);
				#else
					//Read E(k)
					pFuncReadData(ulWriteAddress + (j + k) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xCalComplex1, ALGORITHM_COMPLEX_SIZE);
					//Read O(k)
					pFuncReadData(ulWriteAddress + (j + k + levelCount) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xCalComplex2, ALGORITHM_COMPLEX_SIZE);
					//W * O(k)
					xTempComplex = Complex_Mul(&W, &xCalComplex2);
					
					//Sub  E(k) - W * O(k)
					xCalComplex2 = Complex_Sub(&xCalComplex1, &xTempComplex);
					
					//Add E(k) + W * O(k)
					xCalComplex1 = Complex_Add(&xCalComplex1, &xTempComplex);
					//Store 
					pFuncWriteData(ulWriteAddress + (j + k) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xCalComplex1, ALGORITHM_COMPLEX_SIZE);
					pFuncWriteData(ulWriteAddress + (j + k + levelCount) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xCalComplex2, ALGORITHM_COMPLEX_SIZE);
				#endif
				}
			#if ALGORITHM_EXT_MEMORY_ACCELERATE > 0
				if (levelCount > ACCELERATE_BUFFER_SIZE)
				{
					pFuncWriteData(ulWriteAddress + (j + k - ACCELERATE_BUFFER_SIZE) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateLeftBuffer, ACCELERATE_BUFFER_SIZE * ALGORITHM_COMPLEX_SIZE);
					pFuncWriteData(ulWriteAddress + (j + k - ACCELERATE_BUFFER_SIZE + levelCount) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateRightBuffer, ACCELERATE_BUFFER_SIZE * ALGORITHM_COMPLEX_SIZE);
				}
				else
				{
					pFuncWriteData(ulWriteAddress + j * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateLeftBuffer, levelCount * ALGORITHM_COMPLEX_SIZE);
					pFuncWriteData(ulWriteAddress + (j + levelCount) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateRightBuffer, levelCount * ALGORITHM_COMPLEX_SIZE);
				}
			#endif
			}
		}
	}
	
	return 1;
}

/**
  * @brief  对数据进行IFFT变换
  * @param  pxData: 输入实数数据数组
  * @param  ulSize: 输入数据大小
  * @param  pxOut: 输出傅里叶变换后的结果，大小必须未输入数组的两倍
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @param  pFuncWriteData: 函数指针->外部内存写函数(用于大量数据计算时)
  * @param  ulWriteAddress: 外部内存写开始地址
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE IFFT(Complex *pxFFTData, unsigned long ulSize, ALGORITHM_HANDLE_TYPE *pxOut,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress,
							void (*pFuncWriteData)(unsigned long, unsigned char *, unsigned long), unsigned long ulWriteAddress)
{
	int i, j, n;
	int depth = log2(ulSize);
	int bitReverse, levelCount;
	Complex W, xTempComplex;
	
	//FFT 公式： 前半部分 x(n) = E(n) + W * O(n)
	//           后半部分 x(n) = E(n) - W * O(n)
	if (pxFFTData == NULL && pFuncReadData == NULL)
		return 0;
	
	if (pxFFTData != NULL)
	{
		Complex *pxInComplex;
		Complex *pxOutComplex;
		
		if (pxOut == NULL)
			return 0;
		
		pxInComplex = (Complex *)pxFFTData;
		pxOutComplex = (Complex *)pxOut;
		
		//序列倒序转换
		for (i = 0;i < ulSize;i++)
		{
			bitReverse = 0;
			for (j = 0;j < depth;j++)
			{
				if (i & (1 << j))
				{
					bitReverse |= (1 << (depth - j - 1));
				}
			}
			
			if (bitReverse >= i)
			{
				pxOutComplex[i].fReal = pxInComplex[bitReverse].fReal;
				pxOutComplex[i].fImage = pxInComplex[bitReverse].fImage;
				pxOutComplex[bitReverse].fReal = pxInComplex[i].fReal;
				pxOutComplex[bitReverse].fImage = pxInComplex[i].fImage;
			}
		}
		
		//从下往上推导
		for (i = 0;i < depth;i++)
		{
			//每一级所拥有的组数
			levelCount = (1 << i);
			for (j = 0;j < ulSize;j += 2 * levelCount)
			{
				//对每一组进行蝶形运算
				for (n = 0;n < levelCount;n++)
				{	
					W.fReal = cos(2 * ALGORITHM_PI / ulSize * (ulSize * n / 2 / levelCount));
					W.fImage = sin(2 * ALGORITHM_PI / ulSize * (ulSize * n / 2 / levelCount));
					
					xTempComplex = Complex_Mul(&W, &pxOutComplex[j + n + levelCount]);
					//Sub
					pxOutComplex[j + n + levelCount] = Complex_Sub(&pxOutComplex[j + n], &xTempComplex);
					//Add
					pxOutComplex[j + n] = Complex_Add(&pxOutComplex[j + n], &xTempComplex);
				}
			}
		}
		
		for (i = 0;i < ulSize;i++)
		{
			pxOutComplex[i].fReal /= ulSize;
			pxOutComplex[i].fImage /= ulSize;
		}
	}
	else
	{
	#if !ALGORITHM_EXT_MEMORY_ACCELERATE > 0
		Complex xCalComplex1, xCalComplex2;
	#endif
		Complex xChangePosComplex;
		
		if (pFuncWriteData == NULL)
			return 0;
		
		//序列倒序转换
		for (i = 0;i < ulSize;i++)
		{
			bitReverse = 0;
			for (j = 0;j < depth;j++)
			{
				if (i & (1 << j))
				{
					bitReverse |= (1 << (depth - j - 1));
				}
			}
			
			if (bitReverse >= i)
			{
				//out[bitReverse] = in[i]
				pFuncReadData(ulReadAddress + i * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xChangePosComplex, ALGORITHM_COMPLEX_SIZE); 
				xTempComplex.fReal = xChangePosComplex.fReal;
				xTempComplex.fImage = xChangePosComplex.fImage;
				pFuncWriteData(ulWriteAddress + bitReverse * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xTempComplex, ALGORITHM_COMPLEX_SIZE);
				
				//out[i] = in[bitReverse]
				pFuncReadData(ulReadAddress + bitReverse * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xChangePosComplex, ALGORITHM_COMPLEX_SIZE); 
				xTempComplex.fReal = xChangePosComplex.fReal;
				xTempComplex.fImage = xChangePosComplex.fImage;
				pFuncWriteData(ulWriteAddress + i * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xTempComplex, ALGORITHM_COMPLEX_SIZE);
			}
		}
		
		//从下往上推导
		for (i = 0;i < depth;i++)
		{
			//每一级所拥有的组数
			levelCount = (1 << i);
			for (j = 0;j < ulSize;j += 2 * levelCount)
			{
				//对每一组进行蝶形运算
				for (n = 0;n < levelCount;n++)
				{	
					W.fReal = cos(2 * ALGORITHM_PI / ulSize * (ulSize * n / 2 / levelCount));
					W.fImage = sin(2 * ALGORITHM_PI / ulSize * (ulSize * n / 2 / levelCount));
				#if ALGORITHM_EXT_MEMORY_ACCELERATE > 0
					if (n % ACCELERATE_BUFFER_SIZE == 0)
					{
						if (n != 0)
						{
							pFuncWriteData(ulWriteAddress + (j + n - ACCELERATE_BUFFER_SIZE) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateLeftBuffer, ACCELERATE_BUFFER_SIZE * ALGORITHM_COMPLEX_SIZE);
							pFuncWriteData(ulWriteAddress + (j + n - ACCELERATE_BUFFER_SIZE + levelCount) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateRightBuffer, ACCELERATE_BUFFER_SIZE * ALGORITHM_COMPLEX_SIZE);
						}
						//left
						pFuncReadData(ulWriteAddress + (j + n) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateLeftBuffer, (levelCount > ACCELERATE_BUFFER_SIZE ? ACCELERATE_BUFFER_SIZE : levelCount) * ALGORITHM_COMPLEX_SIZE);
						//right
						pFuncReadData(ulWriteAddress + (j + n + levelCount) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateRightBuffer, (levelCount > ACCELERATE_BUFFER_SIZE ? ACCELERATE_BUFFER_SIZE : levelCount) * ALGORITHM_COMPLEX_SIZE);
					}
					//W * O(k)
					xTempComplex = Complex_Mul(&W, &g_xAccelerateRightBuffer[n % ACCELERATE_BUFFER_SIZE]);
					//Sub  E(k) - W * O(k)
					g_xAccelerateRightBuffer[n % ACCELERATE_BUFFER_SIZE] = Complex_Sub(&g_xAccelerateLeftBuffer[n % ACCELERATE_BUFFER_SIZE], &xTempComplex);
					
					//Add E(k) + W * O(k)
					g_xAccelerateLeftBuffer[n % ACCELERATE_BUFFER_SIZE] = Complex_Add(&g_xAccelerateLeftBuffer[n % ACCELERATE_BUFFER_SIZE], &xTempComplex);
				#else
					//Read E(n)
					pFuncReadData(ulWriteAddress + (j + n) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xCalComplex1, ALGORITHM_COMPLEX_SIZE);
					//Read O(n)
					pFuncReadData(ulWriteAddress + (j + n + levelCount) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xCalComplex2, ALGORITHM_COMPLEX_SIZE);
					//W * O(n)
					xTempComplex = Complex_Mul(&W, &xCalComplex2);
					
					//Sub  E(n) - W * O(n)
					xCalComplex2 = Complex_Sub(&xCalComplex1, &xTempComplex);
					
					//Add E(n) + W * O(n)
					xCalComplex1 = Complex_Add(&xCalComplex1, &xTempComplex);
					//Store 
					pFuncWriteData(ulWriteAddress + (j + n) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xCalComplex1, ALGORITHM_COMPLEX_SIZE);
					pFuncWriteData(ulWriteAddress + (j + n + levelCount) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xCalComplex2, ALGORITHM_COMPLEX_SIZE);
				#endif
				}
			#if ALGORITHM_EXT_MEMORY_ACCELERATE > 0
				if (levelCount > ACCELERATE_BUFFER_SIZE)
				{
					pFuncWriteData(ulWriteAddress + (j + n - ACCELERATE_BUFFER_SIZE) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateLeftBuffer, ACCELERATE_BUFFER_SIZE * ALGORITHM_COMPLEX_SIZE);
					pFuncWriteData(ulWriteAddress + (j + n - ACCELERATE_BUFFER_SIZE + levelCount) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateRightBuffer, ACCELERATE_BUFFER_SIZE * ALGORITHM_COMPLEX_SIZE);
				}
				else
				{
					pFuncWriteData(ulWriteAddress + j * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateLeftBuffer, levelCount * ALGORITHM_COMPLEX_SIZE);
					pFuncWriteData(ulWriteAddress + (j + levelCount) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&g_xAccelerateRightBuffer, levelCount * ALGORITHM_COMPLEX_SIZE);
				}
			#endif
			}
		}
		
		for (i = 0;i < ulSize;i++)
		{
			pFuncReadData(ulWriteAddress + i * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xTempComplex, ALGORITHM_COMPLEX_SIZE);
			xTempComplex.fReal /= ulSize;
			xTempComplex.fImage /= ulSize;
			pFuncWriteData(ulWriteAddress + i * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xTempComplex, ALGORITHM_COMPLEX_SIZE);
		}
	}
	
	return 1;
}

/**
  * @brief  计算频点区间内的最大值
  * @param  pxFFTResult: 输入FFT计算后的结果
  * @param  ulSize: FFT复数结果数量
  * @param  ulSamplingRate: 采样频率
  * @param  fFrequency: 计算频点值
  * @param  fRange: 频点范围 Hz
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @retval ALGORITHM_RESULT_TYPE - 最大复数模值 / size 或最大复数模值 / (size / 2)
  */
ALGORITHM_RESULT_TYPE GetMaxComplexInFreqRange(Complex *pxFFTResult, unsigned long ulSize, unsigned long ulSamplingRate, float fFrequency, float fRange, 
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress)
{
	ALGORITHM_RESULT_TYPE xResult = 0, xCompare;
	Complex xComplex;
	float fResolution = 0;
	float fMinFrequency, fMaxFrequency;
	unsigned long ulMinIndex, ulMaxIndex;
	int i;
	
	if (pxFFTResult != NULL && pFuncReadData != NULL)
		return 0;
	if (ulSize == 0)
		return 0;
	if (fFrequency < 0 || fFrequency > ulSamplingRate / 2.0f)
		return 0;
	
	//频谱分辨率
	fResolution = ulSamplingRate * 1.0f / ulSize;
	
	//频率区间
	fMinFrequency = fFrequency - fRange;
	fMaxFrequency = fFrequency + fRange;
	if (fMinFrequency < 0)
		fMinFrequency = 0;
	if (fMinFrequency > ulSamplingRate / 2.0f)
		fMinFrequency = ulSamplingRate / 2.0f;
	if (fMaxFrequency < 0)
		fMaxFrequency = 0;
	if (fMaxFrequency > ulSamplingRate / 2.0f)
		fMaxFrequency = ulSamplingRate / 2.0f;
	//频率索引
	ulMinIndex = fMinFrequency / fResolution + 0.5f;
	ulMaxIndex = fMaxFrequency / fResolution + 0.5f;
	
	if (pxFFTResult != NULL)
	{		
		for (i = ulMinIndex;i <= ulMaxIndex;i++)
		{
			xCompare = Complex_Cal(&pxFFTResult[i]);
			
			if (i == 0)
				xCompare /= ulSize;
			else
				xCompare /= (ulSize / 2);
			
			if (i == ulMinIndex)
				xResult = xCompare;
			else if (xCompare > xResult)
				xResult = xCompare;
		}
	}
	else
	{
		for (i = ulMinIndex;i <= ulMaxIndex;i++)
		{
			pFuncReadData(ulReadAddress + i * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xComplex, ALGORITHM_COMPLEX_SIZE);
			xCompare = Complex_Cal(&xComplex);
			
			if (i == 0)
				xCompare /= ulSize;
			else
				xCompare /= (ulSize / 2);
			
			if (i == ulMinIndex)
				xResult = xCompare;
			else if (xCompare > xResult)
				xResult = xCompare;
		}
	}
	
	return xResult;
}

/**
  * @brief  计算振动速度(速度均方根) - 10-1000Hz
  * @param  pxFFTResult: 输入FFT计算后的结果
  * @param  ulSize: FFT复数结果数量
  * @param  ulSamplingRate: 采样频率
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE VRMS(Complex *pxFFTResult, unsigned long ulSize, unsigned long ulSamplingRate,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress)
{
	ALGORITHM_RESULT_TYPE xResult = 0, xTemp;
	float fResolution;
	int i;
	unsigned long ulMinIndex, ulMaxIndex;
	
	if (pxFFTResult == NULL && pFuncReadData == NULL)
		return 0;
	if (ulSamplingRate < 2000)
		return 0;
	
	//频谱分辨率
	fResolution = ulSamplingRate * 1.0f / ulSize;
	//10-1000Hz范围
	ulMinIndex = 10 / fResolution + 0.5f;
	ulMaxIndex = 1000 / fResolution + 0.5f;
	
	if (ulMinIndex > (ulSize / 2))
		ulMinIndex = ulSize / 2;
	if (ulMaxIndex > (ulSize / 2))
		ulMaxIndex = ulSize / 2;
	
	
	if (pxFFTResult != NULL)
	{
		for (i = ulMinIndex;i <= ulMaxIndex;i++)
		{
			xTemp = (Complex_Cal(&pxFFTResult[i]) / (ulSize / 2.0f) / 1000.0f * ALGORITHM_G) / (i * fResolution);
			xResult += xTemp * xTemp;
		}
	}
	else
	{
		Complex xComplex;
		
		for (i = ulMinIndex;i <= ulMaxIndex;i++)
		{
			pFuncReadData(ulReadAddress + i * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xComplex, ALGORITHM_COMPLEX_SIZE);
			xTemp = (Complex_Cal(&xComplex) / (ulSize / 2.0f) / 1000.0f * ALGORITHM_G) / (i * fResolution);
			xResult += xTemp * xTemp;
		}
	}
	
	xResult = (float)(1000 / (2 * ALGORITHM_PI) * sqrtf(xResult * 1 / 2));
	
	return xResult;
}

/**
  * @brief  计算高频均方根 - 2-10KHz
  * @param  pxFFTResult: 输入FFT计算后的结果 --- 数据会被改变
  * @param  ulSize: FFT复数结果数量
  * @param  ulSamplingRate: 采样频率
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @param  pFuncWriteData: 函数指针->外部内存写函数(用于大量数据计算时)
  * @param  ulWriteAddress: 外部内存写开始地址
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE HighFrequencyRMS(Complex *pxFFTResult, unsigned long ulSize, unsigned long ulSamplingRate, float *pxIFFTWave,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress,
							void (*pFuncWriteData)(unsigned long, unsigned char *, unsigned long), unsigned long ulWriteAddress)
{
	ALGORITHM_RESULT_TYPE xResult = 0;
	float fResolution;
	int i;
	unsigned long ulMinIndex, ulMaxIndex;
	
	if (pxFFTResult != NULL && pFuncReadData != NULL)
		return 0;
	
	if (ulSamplingRate <= 4000)
		return 0;
	
	//频谱分辨率
	fResolution = ulSamplingRate * 1.0f / ulSize;
	//2000-10000Hz范围
	ulMinIndex = 2000 / fResolution + 0.5f;
	ulMaxIndex = 10000 / fResolution + 0.5f;
	
	if (ulMinIndex > (ulSize / 2))
		ulMinIndex = ulSize / 2;
	if (ulMaxIndex > (ulSize / 2))
		ulMaxIndex = ulSize / 2;
	
	if (pxFFTResult != NULL)
	{
		if (pxIFFTWave == NULL)
			return 0;
		
		//去除直流分量
		pxFFTResult[0].fReal = 0;
		pxFFTResult[0].fImage = 0;
		
		//去除其他频点
		for (i = 1;i < ulMinIndex;i ++)
		{
			pxFFTResult[i].fReal = 0;
			pxFFTResult[i].fImage = 0;
			pxFFTResult[ulSize - i].fReal = 0;
			pxFFTResult[ulSize - i].fImage = 0;
		}
		
		for (i = ulMaxIndex + 1;i <= ulSize / 2;i++)
		{
			pxFFTResult[i].fReal = 0;
			pxFFTResult[i].fImage = 0;
			pxFFTResult[ulSize - i].fReal = 0;
			pxFFTResult[ulSize - i].fImage = 0;
		}
	}
	else
	{
		Complex xTempComplex = {0, 0};
		
		if (pFuncWriteData == NULL)
			return 0;
		
		//去除直流分量
		pFuncWriteData(ulReadAddress, (unsigned char *)&xTempComplex, ALGORITHM_COMPLEX_SIZE);
		
		//去除其他频点
		for (i = 1;i < ulMinIndex;i ++)
		{
			pFuncWriteData(ulReadAddress + i * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xTempComplex, ALGORITHM_COMPLEX_SIZE);
			pFuncWriteData(ulReadAddress + (ulSize - i) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xTempComplex, ALGORITHM_COMPLEX_SIZE);
		}
		
		for (i = ulMaxIndex + 1;i <= ulSize / 2;i++)
		{
			pFuncWriteData(ulReadAddress + i * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xTempComplex, ALGORITHM_COMPLEX_SIZE);
			pFuncWriteData(ulReadAddress + (ulSize - i) * ALGORITHM_COMPLEX_SIZE, (unsigned char *)&xTempComplex, ALGORITHM_COMPLEX_SIZE);
		}
	}
	
	//逆变换
	IFFT(pxFFTResult, ulSize, pxIFFTWave, pFuncReadData, ulReadAddress, pFuncWriteData, ulWriteAddress);
	
	//求均方根
	xResult = RMS(pxIFFTWave, ulSize, pFuncReadData, ulWriteAddress);
	
	return xResult;
}

/**
  * @brief  对所有元素进行偏移
  * @param  pxData: 输入元素数组
  * @param  ulSize: 数量
  * @param  xOffset: 偏移值
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @param  pFuncWriteData: 函数指针->外部内存写函数(用于大量数据计算时)
  * @param  ulWriteAddress: 外部内存写开始地址
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE Add(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize, ALGORITHM_HANDLE_TYPE xOffset, 
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress,
							void (*pFuncWriteData)(unsigned long, unsigned char *, unsigned long), unsigned long ulWriteAddress)
{
	int i;
	
	if (pxData == NULL && pFuncReadData == NULL)
		return 0;
	if (xOffset == 0)
		return 1;
	
	if (pxData != NULL)
	{
		for (i = 0;i < ulSize;i++)
		{
			pxData[i] += xOffset;
		}
	}
	else
	{
		ALGORITHM_HANDLE_TYPE xTempData;
		
		for (i = 0;i < ulSize;i++)
		{
			pFuncReadData(ulReadAddress + i * ALGORITHM_HANDLE_SIZE, (unsigned char *)&xTempData, ALGORITHM_HANDLE_SIZE);
			xTempData += xOffset;
			pFuncWriteData(ulWriteAddress + i * ALGORITHM_HANDLE_SIZE, (unsigned char *)&xTempData, ALGORITHM_HANDLE_SIZE);
		} 
	}
	
	return 1;
}

/**
  * @brief  求出一组数据的阈值 通过z-score分数作为判断标准
  * @param  pxData: 输入数据
  * @param  ulSize: FFT复数结果数量
  * @param  pxHighLimit: 数据上限
  * @param  pxLowLimit: 数据下限 
  * @param  pFuncReadData: 函数指针->外部内存读函数(用于大量数据计算时)
  * @param  ulReadAddress: 外部内存读开始地址
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE CalculateAdaptiveThreshold(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize, 
							ALGORITHM_HANDLE_TYPE *pxHighLimit, ALGORITHM_HANDLE_TYPE *pxLowLimit,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress, float fScale)
{
	int i;
	unsigned char ucSetFlag = 0;
	ALGORITHM_HANDLE_TYPE xHigh = 0, xLow = 0, xTempData, xDiff;
	ALGORITHM_RESULT_TYPE mean, std, zScore;
	
	//Check parameter
	if (pxData == NULL && pFuncReadData == NULL)
		return 0;
	
	if (pxHighLimit == NULL && pxLowLimit == NULL)
		return 0;
	
	if (ulSize == 0 || ulSize == 1)
		return 0;
	
	std = Std(pxData, ulSize, pFuncReadData, ulReadAddress);
	if (std == 0)
		return 0;
	mean = Mean(pxData, ulSize, pFuncReadData, ulReadAddress);
	
	if (pxData != NULL)
	{
		for (i = 0;i < ulSize;i++)
		{
			zScore = (pxData[i] - mean) / std;
			
			//Normal data
			if (zScore > -3 && zScore < 3)
			{
				if (ucSetFlag == 0)
				{
					xHigh = pxData[i];
					xLow = pxData[i];
					ucSetFlag = 1;
				}
				else
				{
					if (pxData[i] > xHigh)
						xHigh = pxData[i];
					
					if (pxData[i] < xLow)
						xLow = pxData[i];
				}
			}
		}
	}
	else
	{
		for (i = 0;i < ulSize;i++)
		{
			pFuncReadData(ulReadAddress + i * ALGORITHM_HANDLE_SIZE, (unsigned char *)&xTempData, ALGORITHM_HANDLE_SIZE);
			zScore = (xTempData - mean) / std;
			
			//Normal data
			if (zScore > -3 && zScore < 3)
			{
				if (ucSetFlag == 0)
				{
					xHigh = xTempData;
					xLow = xTempData;
					ucSetFlag = 1;
				}
				else
				{
					if (xTempData > xHigh)
						xHigh = xTempData;
					
					if (xTempData < xLow)
						xLow = xTempData;
				}
			}
		}
	}
	
	xDiff = xHigh - xLow;
	if (pxHighLimit != NULL)
		*pxHighLimit = xHigh + fScale * xDiff;
	
	if (pxLowLimit != NULL)
		*pxLowLimit = xLow - fScale * xDiff;
	
	return 1;
}

/**
  * @brief  复数计算模值
  * @param  pxComplex: 输入复数 --- 传指针为了加快运算速度
  * @retval ALGORITHM_RESULT_TYPE类型的结果
  */
ALGORITHM_RESULT_TYPE Complex_Cal(Complex *pxComplex)
{
	return sqrt(pxComplex->fReal * pxComplex->fReal + pxComplex->fImage * pxComplex->fImage);
}

/**
  * @brief  复数加法
  * @retval Complex类型的结果
  */
Complex Complex_Add(Complex *pxOp1, Complex *pxOp2)
{
	Complex xResult;
	
	xResult.fReal = pxOp1->fReal + pxOp2->fReal;
	xResult.fImage = pxOp1->fImage + pxOp2->fImage;
	
	return xResult;
}

/**
  * @brief  复数减法
  * @retval Complex类型的结果
  */
Complex Complex_Sub(Complex *pxOp1, Complex *pxOp2)
{
	Complex xResult;
	
	xResult.fReal = pxOp1->fReal - pxOp2->fReal;
	xResult.fImage = pxOp1->fImage - pxOp2->fImage;
	
	return xResult;
}

/**
  * @brief  复数乘法
  * @retval Complex类型的结果
  */
Complex Complex_Mul(Complex *pxOp1, Complex *pxOp2)
{
	Complex xResult;
	
	xResult.fReal = pxOp1->fReal * pxOp2->fReal - pxOp1->fImage * pxOp2->fImage;
	xResult.fImage = pxOp1->fReal * pxOp2->fImage + pxOp1->fImage * pxOp2->fReal;
	
	return xResult;
}
