/* Includes ------------------------------------------------------------------*/
#include "simcom_tcpip.h"

#if SIMCOM > 0
#if CELLUAR_TCPIP_ENABLE > 0
/* Private macros ------------------------------------------------------------*/

/* Extern variables ----------------------------------------------------------*/
extern Celluar_Socket g_xCelluarSocket[];
extern unsigned long g_ulMaxSocketID;

/**
  * @brief  SIMCom get net attach status
  * @param  None
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_GetNetworkAttachStatus()
{
	Celluar_ErrorStatus eResult;
	char cCommand[12], cResponse[48];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CGATT?\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+CGATT:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
	{
		if (!strstr(cResponse, "+CGATT: 1"))
		{
			eResult = Celluar_Error;
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  SIMCom Set IP Head  when received data. +IPD(datalength):
  * @param  None
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_SetIPHead(bool bStatus)
{
	Celluar_ErrorStatus eResult;
	char cCommand[16];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CIPHEAD=%d\r", bStatus ? 1 : 0);
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  SIMCom set net attach status
  * @param  bStatus:true / false
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_NetworkAttach(bool bStatus)
{
	Celluar_ErrorStatus eResult;
	char cCommand[16];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CGATT=%d\r", bStatus ? 1 : 0);
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  SIMCom set apn
  * @param  
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_SetAPN(char *pcAPN, char *pcUser, char *pcPassword)
{
	Celluar_ErrorStatus eResult;
	char cCommand[200];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CSTT=\"%s\",\"%s\",\"%s\"\r", pcAPN ? pcAPN : "", pcUser ? pcUser : "", pcPassword ? pcPassword : "");
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Bring up wireless connection with GPRS or CSD
  * @param  None
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_BringUpWirelessConnection()
{
	Celluar_ErrorStatus eResult;
	char cCommand[12];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CIICR\r");
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_2M);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Get Local IP Address
  * @param  None
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_GetLocalIPAddress()
{
	Celluar_ErrorStatus eResult;
	char cCommand[12];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CIFSR\r");
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), ".", NULL, 0, NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  GPRS Deactivate
  * @param  None		
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_DeactivateGPRS()
{
	Celluar_ErrorStatus eResult;
	char cCommand[16];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CIPSHUT\r");
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "SHUT OK", NULL, 0, NULL, CELLUAR_WAITTIME_1S);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Get socket status
  * @param  pcNetStatus: Return status:
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_GetNetStatus(char *pcNetStatus, unsigned long ulBufferSize)
{
	Celluar_ErrorStatus eResult;
	char cCommand[16], cResponse[200];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CIPSTATUS\r");
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "STATE:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS);
	CutString(pcNetStatus, ulBufferSize, cResponse, "STATE:", "\r\n");
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Connect 
  * @param  ulSocketID
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_Connect(unsigned long ulSocketID)
{
	Celluar_ErrorStatus eResult;
	char cCommand[150], cResponse[48];
	Celluar_Socket *pxSocket;
	
	if (ulSocketID >= g_ulMaxSocketID)
		return Celluar_Error;
	
	pxSocket = &g_xCelluarSocket[ulSocketID];
	
	if (pxSocket->bUsed == false)
		return Celluar_Error;
	
	Celluar_Take();
#if CELLUAR_MULTIIPCONNECTION_ENABLE > 0
	sprintf(cCommand, "AT+CIPSTART=%lu,\"%s\",\"%s\",\"%s\"\r", ulSocketID, pxSocket->cMode, pxSocket->cDestinationHost, pxSocket->cDestinationPort);
#else
	sprintf(cCommand, "AT+CIPSTART=\"%s\",\"%s\",\"%s\"\r", pxSocket->cMode, pxSocket->cDestinationHost, pxSocket->cDestinationPort);
#endif
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "CONNECT", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_2M)) == Celluar_OK)
	{
		if (strstr(cResponse, "CONNECT OK"))
		{
			pxSocket->bStatus = true;
		}
		else
		{
			eResult = Celluar_Error;
		}
	}
#if CELLUAR_RECONNECT_ENABLE > 0
	pxSocket->bReconnectEnable = true;
#endif
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Disconnect 
  * @param  ulSocketID
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_Disconnect(unsigned long ulSocketID)
{
	Celluar_ErrorStatus eResult;
	char cCommand[20];
	Celluar_Socket *pxSocket;
	
	if (ulSocketID >= g_ulMaxSocketID)
		return Celluar_Error;
	
	pxSocket = &g_xCelluarSocket[ulSocketID];
	if (pxSocket->bUsed == false)
		return Celluar_Error;
	
	Celluar_Take();
#if CELLUAR_MULTIIPCONNECTION_ENABLE > 0
	sprintf(cCommand, "AT+CIPCLOSE=%lu,0\r", ulSocketID);
#else
	sprintf(cCommand, "AT+CIPCLOSE=0\r");
#endif
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "CLOSE OK", NULL, 0, NULL, CELLUAR_WAITTIME_500MS);
	pxSocket->bStatus = false;
	Celluar_Give();
	return eResult;
}

/**
  * @brief  send data
  * @param  ulSocketID
  * @param  pucData: pointer to data
  * @param  ulDataSize:  length of data
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_Send(unsigned long ulSocketID, unsigned char *pucData, unsigned long ulDataSize)
{
	Celluar_ErrorStatus eResult = Celluar_Error;
	char cCommand[20], cResponse[48];
	Celluar_Socket *pxSocket;
	
	if (ulSocketID >= g_ulMaxSocketID || pucData == NULL)
		return Celluar_Error;
	
	pxSocket = &g_xCelluarSocket[ulSocketID];
	if (pxSocket->bUsed == false || pxSocket->bStatus == false)
		return Celluar_Error;
	
	Celluar_Take();
	while (ulDataSize)
	{
		unsigned long ulSendSize = ulDataSize > CELLUAR_TCP_UDP_MAX_SIZE ? CELLUAR_TCP_UDP_MAX_SIZE : ulDataSize;
	#if CELLUAR_MULTIIPCONNECTION_ENABLE > 0
		sprintf(cCommand, "AT+CIPSEND=%lu,%lu\r", ulSocketID, ulSendSize);
	#else
		sprintf(cCommand, "AT+CIPSEND=%lu\r", ulSendSize);
	#endif
		if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), ">", NULL, 0, NULL, CELLUAR_WAITTIME_2S)) == Celluar_OK)
		{
			if ((eResult = Celluar_SendCommandAndWaitForResponse((char *)pucData, ulSendSize, "SEND", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_1M)) == Celluar_OK)
			{
				//SEND OK or SEND FAIL
				if (strstr(cResponse, "SEND FAIL") != NULL)
				{
					eResult = Celluar_Error;
				}
			}
		}
		
		if (eResult != Celluar_OK)
			break;
		
		ulDataSize 	-= ulSendSize;
		pucData		+= ulSendSize;
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  get socket status
  * @param  ulSocketID		
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_GetSocketStatus(unsigned long ulSocketID)
{
	Celluar_ErrorStatus eResult;
	char cCommand[20], cResponse[200];
	Celluar_Socket *pxSocket;
#if CELLUAR_MULTIIPCONNECTION_ENABLE > 0
	char cNumber[4];
	char cNetStatus[100];
#endif

	if (ulSocketID >= g_ulMaxSocketID)
		return Celluar_Error;
	
	pxSocket = &g_xCelluarSocket[ulSocketID];

	if (pxSocket->bUsed == false)
		return Celluar_Error;
	Celluar_Take();
	sprintf(cCommand, "AT+CIPSTATUS\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "STATE:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
	{
	#if CELLUAR_MULTIIPCONNECTION_ENABLE > 0
		sprintf(cNumber, "%d", ulSocketID);
		CutString(cNetStatus, sizeof(cNetStatus), cResponse, cNumber, "\r\n");
		if (strstr(cNetStatus, "CONNECT OK"))
		{
			pxSocket->bStatus = true;
		}
		else
		{
			eResult = Celluar_Error;
		}
	#else
		if (strstr(cResponse, "CONNECT OK"))
		{
			pxSocket->bStatus = true;
		}
		else
		{
			eResult = Celluar_Error;
		}
	#endif
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  set ssl
  * @param  bStatus: on or off			
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_SetSSL(bool bStatus)
{
	Celluar_ErrorStatus eResult;
	char cCommand[16];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CIPSSL=%d\r", bStatus == true ? 1 : 0);
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_1S);
	Celluar_Give();
	return eResult;
}

#if CELLUAR_MULTIIPCONNECTION_ENABLE > 0
/**
  * @brief  start multi ip connection
  * @param  bStatus: on or off			
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus SIMCom_StartUpMultiIPConnection(bool bStatus)
{
	Celluar_ErrorStatus eResult;
	char cCommand[16];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CIPMUX=%d\r", bStatus);
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();
	return eResult;
}
#endif

#endif
#endif
