/**
******************************************************************************
* @file         w5100s_conf.c                                                                                              
* @version      V1.0                          
* @date         2018-06-18             
* @brief        配置MCU，移植w5100s程序需要修改的文件，配置w5100s的MAC和IP地址   
*        
* @company      深圳炜世科技有限公司
* @information  WIZnet W5100s官方代理商，全程技术支持，价格优势大！
* @website      www.wisioe.com   
* @forum        www.w5100s.com
* @qqGroup      579842114                                                     
******************************************************************************
*/

#include <stdio.h> 
#include <string.h>
#include "w5100s_conf.h"
#include "w5100s.h"
#include "socket.h"
#include "dhcp.h"
#include "driver/gpio.h"
#include "spi.h"
#include "esp_rom_gpio.h"
#if CONFIG_W5100S_SPI_ETHERNET > 0
    #include "eth.h"
#endif
#include "wit.h"

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/queue.h"

#define ETH_INTN_IO  14

uint8 reboot_flag;
CONFIG_MSG  ConfigMsg;
/* NET */
wiz_NetInfo gWIZNETINFO;

uint8_t ethBuf[ETH_MAX_BUF_SIZE];



void ETHERNET_Init(bool *bModuleStatus, bool *bCableStatus)
{
	unsigned char ucrepeat = 10;
	reset_break_gpio_init();
	spi_init(SPI_PRESCALER_4);
	reg_wizchip_spi_cbfunc(spi_read_byte,spi_send_byte);          // SPI读写数据函数映射
	reg_wizchip_cs_cbfunc(cs_low,cs_high); 
	  
	wizchip_initialize(bModuleStatus, bCableStatus);
	
	if(*bModuleStatus == true)
		return;

	if (gWIZNETINFO.dhcp == NETINFO_DHCP) // DHCP
	{
		wizchip_dhcp_init();
		printf("\r\n----------dhcp init--------------\r\n");
		while (ucrepeat--) 
		{
			if(DHCP_run() == DHCP_IP_LEASED)
			 	break;
			wiz_delay_ms(100);
		}
	}
	else // static
	{

		ctlnetwork(CN_SET_NETINFO, &gWIZNETINFO);
		printf("\r\n----------STATIC Net Information--------------\r\n");
		print_network_information();
		vTaskDelay(2000 / portTICK_PERIOD_MS);
	}  
	// hw_ping_init();
    #if CONFIG_W5100S_SPI_ETHERNET > 0
	    if_eth_task_init();
    #endif
}

/**
*@brief  初始化W5100s的复位与中断管脚
*@param  无
*@return 无
*/
void reset_break_gpio_init(void)
{
  gpio_config_t io_conf = {};
  io_conf.intr_type = GPIO_INTR_DISABLE;
  io_conf.mode = GPIO_MODE_INPUT;
  io_conf.pin_bit_mask = 1 << 14;
  io_conf.pull_down_en = GPIO_PULLDOWN_ENABLE;
  io_conf.pull_up_en = GPIO_PULLUP_DISABLE;
  gpio_config(&io_conf);
  // RSTn 引脚
  esp_rom_gpio_pad_select_gpio(9);   // 选择个GPIO
  gpio_set_direction(9, GPIO_MODE_OUTPUT); // 把这个GPIO作为输出     
  gpio_set_level(9, 1); 
}

/**
*@brief  W5100s复位设置函数
*@param  无
*@return 无
*/
void wizchip_reset(void)
{
  gpio_set_level(9, 0);
  vTaskDelay(1000/portTICK_PERIOD_MS);
  gpio_set_level(9, 1);
  vTaskDelay(1000/portTICK_PERIOD_MS);
}

void wizchip_check(bool *bStatus)
{
    /* Read version register */
    if (getVER() != 0x51) // W5100S
    {
		// W5100S 错误
        printf(" ACCESS ERR : VERSIONR != 0x51, read value = 0x%02x\n", getVER());
		if(bStatus != NULL)
			*bStatus = true;
    }
}

void wizchip_initialize(bool *bModuleStatus, bool *bCableStatus)
{
	uint8_t W5100S_AdrSet[2][4]= {{2,2,2,2},{2,2,2,2}};
    uint8_t tmp1, tmp2;
	intr_kind temp = IK_DEST_UNREACH;

    char repeat = 6;
	if (ctlwizchip(CW_INIT_WIZCHIP, (void*)W5100S_AdrSet) == -1)
		printf(">>>>W5100s memory initialization failed\r\n");

	if(ctlwizchip(CW_SET_INTRMASK,&temp) == -1)
		printf("W5100S interrupt\r\n");

	wizchip_reset();

	wizchip_check(bModuleStatus);

	if(bModuleStatus != NULL)
	{
		if(*bModuleStatus == true)
			return;
	}
	while(repeat--)
	{
		ctlwizchip(CW_GET_PHYLINK, &tmp1);
		ctlwizchip(CW_GET_PHYLINK, &tmp2);
		if(tmp1 == PHY_LINK_ON && tmp2 == PHY_LINK_ON) 
		{
			if(bCableStatus != NULL)
			  *bCableStatus = true;
			break;
		}
		vTaskDelay(500 / portTICK_PERIOD_MS);
	}
}

void print_network_information(void)
{
	wizchip_getnetinfo(&gWIZNETINFO);

	printf("Mac address: %02x:%02x:%02x:%02x:%02x:%02x\n\r",gWIZNETINFO.mac[0],gWIZNETINFO.mac[1],gWIZNETINFO.mac[2],gWIZNETINFO.mac[3],gWIZNETINFO.mac[4],gWIZNETINFO.mac[5]);
	printf("IP address : %d.%d.%d.%d\n\r",gWIZNETINFO.ip[0],gWIZNETINFO.ip[1],gWIZNETINFO.ip[2],gWIZNETINFO.ip[3]);
	printf("SM Mask	   : %d.%d.%d.%d\n\r",gWIZNETINFO.sn[0],gWIZNETINFO.sn[1],gWIZNETINFO.sn[2],gWIZNETINFO.sn[3]);
	printf("Gate way   : %d.%d.%d.%d\n\r",gWIZNETINFO.gw[0],gWIZNETINFO.gw[1],gWIZNETINFO.gw[2],gWIZNETINFO.gw[3]);
	printf("DNS Server : %d.%d.%d.%d\n\r",gWIZNETINFO.dns[0],gWIZNETINFO.dns[1],gWIZNETINFO.dns[2],gWIZNETINFO.dns[3]);
}

/* DHCP */
static void wizchip_dhcp_assign(void)
{
	wiz_NetInfo   *eth_info = NULL;
    getIPfromDHCP(gWIZNETINFO.ip);
    getGWfromDHCP(gWIZNETINFO.gw);
    getSNfromDHCP(gWIZNETINFO.sn);
    getDNSfromDHCP(gWIZNETINFO.dns);
    gWIZNETINFO.dhcp = NETINFO_DHCP;

    ctlnetwork(CN_SET_NETINFO, &gWIZNETINFO);
	printf("\r\n----------DHCP Net Information--------------\r\n");
#if CONFIG_W5100S_SPI_ETHERNET > 0
	eth_info = get_gw_eth_info();
#endif
    print_network_information();
	memcpy(eth_info->mac, gWIZNETINFO.mac, sizeof(gWIZNETINFO.mac));
    memcpy(eth_info->ip, gWIZNETINFO.ip, sizeof(gWIZNETINFO.ip));
    memcpy(eth_info->gw, gWIZNETINFO.gw, sizeof(gWIZNETINFO.gw));
    memcpy(eth_info->sn, gWIZNETINFO.sn, sizeof(gWIZNETINFO.sn));
	WIT_SetUpdateConfigureAreaFlag(true);
}

static void wizchip_dhcp_conflict(void)
{
	printf("wizchip_dhcp_conflict\r\n");
}

void wizchip_dhcp_init(void)
{
    DHCP_init(SOCKET_DHCP, ethBuf);
    reg_dhcp_cbfunc(wizchip_dhcp_assign, wizchip_dhcp_assign, wizchip_dhcp_conflict);
}

char get_dhcp_mode(void)
{
	return  gWIZNETINFO.dhcp;
}

void hw_ping_init(void)
{
    setSLRTR(0x03E8);       // 重传超时时间设置：1000 * 100us
    setSLRCR(3);            // 重传次数设置：3次
    /* 使用中断时可开启该配置 */	
    setSLIMR(0x05);         // 中断开启 PING & TIMEOUT
    setPINGSEQR(0x03E8);    // 请求数据包序列号设置：1000
    setPINGIDR(0x0100);     // 请求数据包ID：100
}

bool get_eth_int_gpio_status(void)
{
	return gpio_get_level(14) ? true : false;
}

char *get_gw_ip(void)
{
	return (char *)gWIZNETINFO.gw;
}

void set_wiz_NetInfo(wiz_NetInfo *netInfo)
{
	memcpy(gWIZNETINFO.mac, netInfo->mac, sizeof(netInfo->mac));	
	memcpy(gWIZNETINFO.ip, netInfo->ip, sizeof(netInfo->ip));	
	memcpy(gWIZNETINFO.sn, netInfo->sn, sizeof(netInfo->sn));	
	memcpy(gWIZNETINFO.gw, netInfo->gw, sizeof(netInfo->gw));	
	memcpy(gWIZNETINFO.dns, netInfo->dns, sizeof(netInfo->dns));	
	gWIZNETINFO.dhcp = netInfo->dhcp;
	printf("store.......\n\r");
	printf("Mac address: %02x:%02x:%02x:%02x:%02x:%02x\n\r",gWIZNETINFO.mac[0],gWIZNETINFO.mac[1],gWIZNETINFO.mac[2],gWIZNETINFO.mac[3],gWIZNETINFO.mac[4],gWIZNETINFO.mac[5]);
	printf("IP address : %d.%d.%d.%d\n\r",gWIZNETINFO.ip[0],gWIZNETINFO.ip[1],gWIZNETINFO.ip[2],gWIZNETINFO.ip[3]);
	printf("SM Mask	   : %d.%d.%d.%d\n\r",gWIZNETINFO.sn[0],gWIZNETINFO.sn[1],gWIZNETINFO.sn[2],gWIZNETINFO.sn[3]);
	printf("Gate way   : %d.%d.%d.%d\n\r",gWIZNETINFO.gw[0],gWIZNETINFO.gw[1],gWIZNETINFO.gw[2],gWIZNETINFO.gw[3]);
	printf("DNS Server : %d.%d.%d.%d\n\r",gWIZNETINFO.dns[0],gWIZNETINFO.dns[1],gWIZNETINFO.dns[2],gWIZNETINFO.dns[3]);
}

wiz_NetInfo * get_wiz_NetInfo(void)
{
	return &gWIZNETINFO;
}