/* Includes ------------------------------------------------------------------*/
#include "celluar.h"
#include "celluar_event.h"
#include "celluar_socket.h"
#include "seq_queue.h"
#include "esp_log.h"
#include "pwr_gpio.h"
#include "celluar_usb.h"

/* Tasks info ----------------------------------------------------------------*/
#define CELLUAR_POLL_TASK_STK				(configMINIMAL_STACK_SIZE * 4)
#define CELLUAR_EVENT_HANDLER_TASK_STK		(configMINIMAL_STACK_SIZE * 2)

#define CELLUAR_RECONNECT_TASK_STK			(configMINIMAL_STACK_SIZE * 4)
#define CELLUAR_MQTT_QUEUE_SIZE				CELLUAR_BUFFER_SIZE

#define CELLUAR_MAX_TCPIP_TRY_COUNT			2

/* Private types -------------------------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
static void						Celluar_Poll(void);
static void 					Celluar_EventPoll(void);
static void						Celluar_Select(void);

/*Task function*/
static void Celluar_PollTask(void *pvParameter);
static void Celluar_EventHandlerTask(void *pvParameter);
#if CELLUAR_RECONNECT_ENABLE > 0
static void Celluar_ReconnectTask(void *pvParameter);
#endif

/* Private variables ---------------------------------------------------------*/
static unsigned char *g_pucDataBuffer;
static unsigned long g_ulBufferLen;

/* Variables -----------------------------------------------------------------*/
#if CELLUAR_PROTOCOL_MQTT_ENABLE > 0
	SeqQueue g_xSocketQueue[CELLUAR_SOCKET_MAX_COUNT];
	void (*pFuncMQTTStatusChangedHandler)(unsigned long, bool);
	bool (*pFuncMQTTReconnectHandler)(unsigned long);
#endif

/* Extern variables ----------------------------------------------------------*/
extern Celluar 			g_xCelluar;

/**
  * @brief  Celluar poll when no module type selected
  * @param  None
  * @retval None
  */
void Celluar_Poll()
{
	xSemaphoreTake(g_xCelluar.xReceivedSemHandle, portMAX_DELAY);
	if (g_xCelluar.bWaiting)
		xSemaphoreGive(g_xCelluar.xResponseSemHandle);
	else
		Celluar_QueuePop(&g_xCelluar.xReceiveQueue);
}

/**
  * @brief  Celluar event poll
  * @param  None
  * @retval None
  */
void Celluar_EventPoll()
{
	Celluar_EventType eEvent;
	if (Celluar_EventGet(&eEvent, &g_pucDataBuffer, &g_ulBufferLen))
	{
	#if CELLUAR_TCPIP_ENABLE > 0
		unsigned long i;
	#endif
		switch(eEvent)
		{
			case Celluar_EventPowerDown:
				g_xCelluar.bIsPowerOn = false;
				g_xCelluar.bHasSIMCard = false;
				g_xCelluar.bHasNetwork = false;
				Celluar_EventPowerDownHandler();
				break;
			#if CELLUAR_TCPIP_ENABLE > 0
			case Celluar_EventNetClosed:
				if (g_ulBufferLen > 0)
				{
					unsigned char ucClosedSocketID = g_pucDataBuffer[0];
					if (ucClosedSocketID < g_ulMaxSocketID)
					{
						g_xCelluarSocket[ucClosedSocketID].bStatus = false;
						Celluar_EventNetClosedHandler(ucClosedSocketID);
					}
				}
				break;
			case Celluar_EventNetReceived:
				if (g_ulBufferLen > 0)
				{
					unsigned char ucReceivedSocketID = g_pucDataBuffer[0];
					if (ucReceivedSocketID < g_ulMaxSocketID)
					{
						Celluar_EventNetReceivedHandler(g_pucDataBuffer[0], g_pucDataBuffer + 1, g_ulBufferLen - 1);
					}
				}
				break;
			#endif
			case Celluar_EventPullCard:
				g_xCelluar.bHasSIMCard = false;
				g_xCelluar.bHasNetwork = false;
			#if CELLUAR_TCPIP_ENABLE > 0
				for (i = 0; i < g_ulMaxSocketID; i++)
				{
					g_xCelluarSocket[i].bStatus = false;
				}
				Celluar_EventPullCardHandler();
			#endif
				break;
			case Celluar_EventPDPDeact:
			#if CELLUAR_TCPIP_ENABLE > 0
				for (i = 0; i < g_ulMaxSocketID; i++)
				{
					g_xCelluarSocket[i].bStatus = false;
				}
			#endif
				Celluar_EventPDPDeactHandler();
				break;
		}
		//Free event data
		 Celluar_EventDel();
	}
}

/**
  * @brief  Celluar module restart and confirm the type of celluar by sending ATI command
  * @param  None
  * @retval true / false
  */
bool Celluar_ModuleInit(bool *pbErrorOccured)
{
#if CELLUAR_RECONNECT_ENABLE > 0
	unsigned char ucError = 0;
#endif
	if (pFuncCelluar_Poll == NULL)
	{
		UBaseType_t uxTaskPriority = uxTaskPriorityGet(NULL);
		int i;
	#if CELLUAR_QUEUE_IS_MALLOC	> 0
		for (i = 0;i < CELLUAR_QUEUE_SIZE;i++)
		{
			g_xCelluar.xReceiveQueue.xReceiveBuffer[i].ucBuffer = (unsigned char *) malloc(CELLUAR_BUFFER_SIZE * sizeof(unsigned char));
		}
	#endif
	#if CELLUAR_PROTOCOL_MQTT_ENABLE > 0
		//Create mqtt queue
		for (i = 0;i < CELLUAR_SOCKET_MAX_COUNT;i++)
		{
			g_xSocketQueue[i] = SeqQueue_Create(CELLUAR_MQTT_QUEUE_SIZE);
		}
	#endif
		
		//Create tasks
	#if CELLUAR_RECONNECT_ENABLE > 0
		xTaskCreate(Celluar_ReconnectTask, "Celluar Reconnect", CELLUAR_RECONNECT_TASK_STK, NULL, uxTaskPriority, NULL);
	#endif
		uxTaskPriority = (uxTaskPriority + 2) > configMAX_PRIORITIES ? configMAX_PRIORITIES : (uxTaskPriority + 2);  
		xTaskCreate(Celluar_PollTask, "Celluar Poll", CELLUAR_POLL_TASK_STK, NULL, uxTaskPriority, NULL);
		xTaskCreate(Celluar_EventHandlerTask, "Celluar EventHandler", CELLUAR_EVENT_HANDLER_TASK_STK, NULL, uxTaskPriority, NULL);
	}
	//Default poll function
	if (pFuncCelluar_Poll == NULL)
		pFuncCelluar_Poll = Celluar_Poll;
#if CELLUAR_RECONNECT_ENABLE > 0
try_again:
#endif
	//PowerOn
	if (Celluar_PowerOnCheckAT())
	{
	#if DEBUG > 0
		Celluar_SetEchoMode(true);
	#else
		Celluar_SetEchoMode(false);
	#endif
		if (g_xCelluar.eModule == Celluar_Undefined)
			Celluar_Select();
		if (g_xCelluar.eModule != Celluar_Undefined)
		{
			pFuncCelluar_BasicSettings();
		#if CELLUAR_TCPIP_ENABLE > 0
			pFuncCelluar_TCPIPInit(&g_xCelluar.ulCurrentAPNIndex);
		#endif	
			*pbErrorOccured = false;
			return true;
		}
	}
	else
	{
		ESP_LOGD("AT ", "Can not power on celluar module.");
	}

#if CELLUAR_RECONNECT_ENABLE > 0
		ucError++;
		if (ucError >= 2)
		{
			*pbErrorOccured = true;
		}
	goto try_again;
#else
	return false;
#endif
}

#if CELLUAR_PROTOCOL_MQTT_ENABLE > 0
/**
  * @brief  Celluar set mqttstatus changed call back
  * @param  None
  * @retval None
  */
void Celluar_SetMQTTStatusChangedCallBack(void (*pFuncMQTTStatusChangedCallBack)(unsigned long, bool))
{
	pFuncMQTTStatusChangedHandler = pFuncMQTTStatusChangedCallBack;
}

#if CELLUAR_RECONNECT_ENABLE > 0
/**
  * @brief  Celluar set mqtt reconnect call back
  * @param  None
  * @retval None
  */
void Celluar_SetMQTTReconnectCallBack(bool (*pFuncMQTTReconnectCallBack)(unsigned long))
{
	pFuncMQTTReconnectHandler = pFuncMQTTReconnectCallBack;
}
#endif

/**
  * @brief  Celluar receive data in buffer
  * @param  None
  * @retval None
  */
long Celluar_Recv(unsigned long ulSocketID, unsigned char *pucBuffer, unsigned long ulSize)
{
	unsigned long ulGetSize = 0;
	
	if (g_xCelluarSocket[ulSocketID].bStatus == false)
		return -1;
	
	if (pucBuffer == NULL || ulSocketID >= g_ulMaxSocketID)
		return 0;
	
	if (SeqQueue_IsEmpty(&g_xSocketQueue[ulSocketID]))
		return 0;
	
	while (ulSize--)
	{
		if (SeqQueue_DeQueue(&g_xSocketQueue[ulSocketID], pucBuffer++) == false)
		{
			break;
		}
		ulGetSize++;
	}
	
	return ulGetSize;
}

/**
  * @brief  Celluar clear seqqueue
  * @param  None
  * @retval None
  */
void Celluar_ClearSeqQueue(unsigned long ulSocketID)
{
	if (ulSocketID < g_ulMaxSocketID)
		SeqQueue_Clear(&g_xSocketQueue[ulSocketID]);
}
#endif

/**
  * @brief  Celluar select module type (SIMCom Quectel)
  * @param  None
  * @retval None
  */
static void Celluar_Select()
{
	char cModuleInfo[60] = {0};
	unsigned char ucRetry = 5;
	
	while (ucRetry > 0)
	{
		if (Celluar_GetModuleInfo(cModuleInfo, sizeof(cModuleInfo)) == Celluar_OK)
		{
			if (strstr(cModuleInfo, "SIM"))
			{
			#if DEBUG > 0
				Debug_Print("Module: SIMCom\r\n");
			#endif
				g_xCelluar.eModule = Celluar_SIMCom;
			}
			else if (strstr(cModuleInfo, "Quectel"))
			{
			#if DEBUG > 0
				Debug_Print("Module: Quectel\r\n");
			#endif
				g_xCelluar.eModule = Celluar_Quectel;
			}
			else
			{
			#if DEBUG > 0
				char cModule[60];
				sprintf(cModule, "Module error info: %s\r\n", cModuleInfo);
				Debug_Print(cModule);
			#endif
			}
			
			if (g_xCelluar.eModule != Celluar_Undefined)
				Celluar_SetFunc(g_xCelluar.eModule);
		}
		else
		{
			//power down(maybe)
		#if DEBUG > 0
			Debug_Print("Error on getting module info.\r\n");
		#endif
			break ;
		}
		
		//Got
		if (g_xCelluar.eModule != Celluar_Undefined)
			break;
		
		ucRetry--;
		vTaskDelay(200);
	}
}

/**
  * @brief  Celluar message receive and handler
  * @param  
  * @retval None
  */
static void Celluar_PollTask(void * pvParameter)
{	
	while (1)
	{
		if (pFuncCelluar_Poll != NULL)
		{
			pFuncCelluar_Poll();
		}
		else
		{
			vTaskDelay(1);
		}
	}
}

/**
  * @brief  Celluar event handler
  * @param  
  * @retval None
  */
static void Celluar_EventHandlerTask(void *pvParameter)
{
	while (1)
	{
		Celluar_EventPoll();
		vTaskDelay(10);
	}
}

#if CELLUAR_RECONNECT_ENABLE > 0
/**
  * @brief  Celluar reconnect task
  * @retval None
  */
static void Celluar_ReconnectTask(void *pvParameter)
{
	while (1)
	{
		int i;
		static unsigned char ucError = 0;
		
		for (i = 0;i < g_ulMaxSocketID;i++)
		{
			if (g_xCelluarSocket[i].bUsed == true && g_xCelluarSocket[i].bReconnectEnable == true && g_xCelluarSocket[i].bStatus == false)
			{
				if (Celluar_IsPowerOn() == true && Celluar_HasSIMCard() == true)
				{
					// printf("disconnect\r\n");
					//disconnect
					if (pFuncCelluar_Disconnect(i) == Celluar_Timeout)
					{
						g_xCelluar.bIsPowerOn = Celluar_IsPowerOnWithAT();
						continue;
					}		
					//connect
					if (Celluar_IsNetAttached() && pFuncCelluar_Connect(i) == Celluar_OK)
					{
						unsigned char ucReconnectedSocketID = i;
						//reset error count
//						ucError = 0;
						//reconnect success event
					#if CELLUAR_PROTOCOL_MQTT_ENABLE > 0 
						if (pFuncMQTTReconnectHandler != NULL)
						{	
							if (pFuncMQTTReconnectHandler(ucReconnectedSocketID) == true)
							{
								ucError = 0;
							}
							else
							{
								ucError++;
							}
						}
					#else
						ucError = 0;
					#endif
					}
					else
					{
						ucError++;
						if (Celluar_IsNetAttached() == false)
						{
							pFuncCelluar_TCPIPInit(&g_xCelluar.ulCurrentAPNIndex);
						}
					}
					
					if (ucError >= CELLUAR_MAX_TCPIP_TRY_COUNT)
					{
						ucError = 0;
						//Restart
						Celluar_IsSoftReboot();
						vTaskDelay(2000); 
						Celluar_Restart();
						if (Celluar_IsPowerOn())
						{
							pFuncCelluar_BasicSettings();
							pFuncCelluar_TCPIPInit(&g_xCelluar.ulCurrentAPNIndex);
						}
					}
				}
				else
				{
					//Restart
					Celluar_IsSoftReboot();
					vTaskDelay(2000);
					Celluar_Restart();
					if (Celluar_IsPowerOn())
					{
						// printf("enter tcp reconnect\r\n");
						pFuncCelluar_BasicSettings();
						pFuncCelluar_TCPIPInit(&g_xCelluar.ulCurrentAPNIndex);
					}
				}
			}
		}
		vTaskDelay(10);
	}
}
#endif
